extern unsigned _stklen = 60000;

#include <math.h>
#include <stdio.h>
#include <string.h>

const long MAX_BUILDINGS_COUNT = 100; //100
const long MAX_SHELTERS_COUNT = 100; //100
const long INFINITY = 1000000000; //1000000000

struct tPoint2D
{
	short y,x;
	void Read(void)
	{
		scanf("%hd %hd", &x, &y);
	}
	long DistTo(const tPoint2D& point)const
	{
		return labs(x - point.x) + labs(y - point.y);
	}
};

int main(void)
{
	static short shelterFreeSpace[MAX_SHELTERS_COUNT];
	static short evacPlan[MAX_BUILDINGS_COUNT][MAX_SHELTERS_COUNT];
	static long manMoveMinCost[MAX_SHELTERS_COUNT][MAX_SHELTERS_COUNT];
	tPoint2D buildingCoords[MAX_BUILDINGS_COUNT];
	tPoint2D shelterCoords[MAX_SHELTERS_COUNT];
	short prevMove[MAX_SHELTERS_COUNT][MAX_SHELTERS_COUNT];
	short minBuilding[MAX_SHELTERS_COUNT][MAX_SHELTERS_COUNT];

	freopen("evacuate.in", "r", stdin);
	freopen("evacuate.out", "w", stdout);

	long iBuilding, cBuildings, iShelter, cShelters;
	scanf("%ld %ld", &cBuildings, &cShelters);
	for(iBuilding = 0; iBuilding < cBuildings; ++iBuilding)
	{
		buildingCoords[iBuilding].Read();
		scanf("%*ld");
	}
	for(iShelter = 0; iShelter < cShelters; ++iShelter)
	{
		shelterCoords[iShelter].Read();
		scanf("%hd", &shelterFreeSpace[iShelter]);
	}

	memset(evacPlan, 0, sizeof(evacPlan));
	for(iBuilding = 0; iBuilding < cBuildings; ++iBuilding)
		for(iShelter = 0; iShelter < cShelters; ++iShelter)
		{
			scanf("%hd", &evacPlan[iBuilding][iShelter]);
			shelterFreeSpace[iShelter] -= evacPlan[iBuilding][iShelter];
		}

	long iSrcShelter, iDestShelter, newMoveCost;
	for(iSrcShelter = 0; iSrcShelter < cShelters; ++iSrcShelter)
		for(iDestShelter = 0; iDestShelter < cShelters; ++iDestShelter)
		{
			manMoveMinCost[iSrcShelter][iDestShelter] = INFINITY;
			prevMove[iSrcShelter][iDestShelter] = iSrcShelter;
			for(iBuilding = 0; iBuilding < cBuildings; ++iBuilding)
				if(evacPlan[iBuilding][iSrcShelter])
				{
					newMoveCost = 
						buildingCoords[iBuilding].DistTo(shelterCoords[iDestShelter]) -
						buildingCoords[iBuilding].DistTo(shelterCoords[iSrcShelter]);
					if(manMoveMinCost[iSrcShelter][iDestShelter] > newMoveCost)
					{
						manMoveMinCost[iSrcShelter][iDestShelter] = newMoveCost;
						minBuilding[iSrcShelter][iDestShelter] = iBuilding;
					}
				}
		}

	long iMidShelter;
	for(iMidShelter = 0; iMidShelter < cShelters; ++iMidShelter)
		for(iSrcShelter = 0; iSrcShelter < cShelters; ++iSrcShelter)
			for(iDestShelter = 0; iDestShelter < cShelters; ++iDestShelter)
			{
				newMoveCost = manMoveMinCost[iSrcShelter][iMidShelter] +
					manMoveMinCost[iMidShelter][iDestShelter];
				if(newMoveCost < manMoveMinCost[iSrcShelter][iDestShelter])
				{
					manMoveMinCost[iSrcShelter][iDestShelter] = newMoveCost;
					prevMove[iSrcShelter][iDestShelter] = prevMove[iMidShelter][iDestShelter];
				}
				if(iSrcShelter == iDestShelter && newMoveCost < 0)
					goto FloydEnd;
			}

FloydEnd:

	if(iMidShelter == cShelters)
		for(iSrcShelter = 0; iSrcShelter < cShelters; ++iSrcShelter)
			for(iDestShelter = 0; iDestShelter < cShelters; ++iDestShelter)
				if(shelterFreeSpace[iDestShelter] &&
					manMoveMinCost[iSrcShelter][iDestShelter] < 0)
						goto SearchEnd;

SearchEnd:

	if(iSrcShelter == cShelters)puts("OPTIMAL");
	else
	{
		long iLastDestShelter;
		do
		{
			iLastDestShelter = iDestShelter;
			iDestShelter = prevMove[iSrcShelter][iDestShelter];
			evacPlan[minBuilding[iDestShelter][iLastDestShelter]][iDestShelter]--;
			evacPlan[minBuilding[iDestShelter][iLastDestShelter]][iLastDestShelter]++;
		}
		while(iDestShelter != iSrcShelter);

		puts("SUBOPTIMAL");
		for(iBuilding = 0; iBuilding < cBuildings; ++iBuilding)
		{
			for(iShelter = 0; iShelter < cShelters; ++iShelter)
			{
				if(iShelter)putchar(' ');
				printf("%hd", evacPlan[iBuilding][iShelter]);
			}
			putchar('\n');
		}
	}


	fclose(stdout);
	fclose(stdin);

	return 0;
}